/************************************************************************
*  NelderMead.h (Minimization Method of Nelder and Mead)
*
*  Version:                     1.1
*  Date:                        9/29/93
*  Author:                      O. Knueppel
*************************************************************************/

#ifndef __NELDERMEAD__
#define __NELDERMEAD__

#include "matrix.h"

VOID NelderMead_Expert (MATRIX & p, VECTOR & y, REAL tol, REAL abstol,
			INT & pok, INT niter, PREAL parms,
			REAL (*pfunc)(VECTOR &));
/*
 * Minimization method of Nelder and Mead (expert version)
 * -------------------------------------------------------
 *
 * INPUTS:
 *
 *   p       real matrix
 *           (row dimension =  (dim + 1), column dimension = dim)
 *           each row contains one point of the amoeba
 *
 *   y       real vector (dimension = dim + 1)
 *           containing the function values of each amoeba point
 *
 *   tol     relative termination tolerance
 *
 *   abstol  absolute termination tolerance
 *
 *   ok      reference to flag (see OUTPUTS)
 *
 *   niter   max. number of iterations
 *
 *   parms   real array (dimension = 3) containing the
 *           parameters of the minimization method:
 *
 *           parms[0] := expansion factor for new trial point
 *           parms[1] := shrink factor
 *           parms[2] := expansion factor
 *
 *           useful values are e.g. parms = { 1.0, 0.5, 2.0 }
 *
 *   pfunc   address of function to be minimized
 *           (parameter:    real vector (dimension = dim)
 *            return value: function value at the point given by
 *                          the parameter)
 *
 * OUTPUTS:
 *
 *   p       real matrix
 *           (row dimension =  (dim + 1), column dimension = dim)
 *           each row contains one point of the amoeba
 *
 *   y       real vector (dimension = dim + 1)
 *           containing the function values of each amoeba point
 *
 *   ok      1, if the algorithm terminated normally
 *           0, if the maximum number of iterations is exceeded
 *
 * RETURN VALUE:
 *
 *           none
 */

REAL NelderMead (VECTOR & x, VECTOR & diam,
		 REAL tol, REAL abstol, INT & ok,
		 INT niter, REAL (*pfunc)(VECTOR &));
/*
 * Minimization method of Nelder and Mead (simple version)
 * -------------------------------------------------------
 *
 * INPUTS:
 *
 *   x       real vector (dimension = dim)
 *           containing a start point
 *
 *   diam    real vector (dimension = dim)
 *           with minimum contained in (x +/- (diam/2))
 *
 *   tol     relative termination tolerance
 *
 *   abstol  absolute termination tolerance
 *
 *   ok      reference to flag (see OUTPUTS)
 *
 *   niter   max. number of iterations
 *
 *   pfunc   address of function to be minimized
 *           (parameter:    real vector (dimension = dim)
 *            return value: function value at the point given by
 *                          the parameter)
 *
 * OUTPUTS:
 *
 *   x       real vector (dimension = dim)
 *           approximation of the minimum point
 *
 *   ok      1, if the algorithm terminated normally
 *           0, if the maximum number of iterations is exceeded
 *
 * RETURN VALUE:
 *
 *           approximation of the minimum value
 */

#endif /* __NELDERMEAD__ */
